/*
 * sync.js
 *
 * Copyright (c) 2005 by Daniel Potter.  All rights reserved.
 *
 * This file is part of the Vana'diel Clock.
 *
 * The Vana'diel Clock is free software; you can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The Vana'diel Clock is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the Vana'diel Clock; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Functions to help synchronize the clock.
 */

var winHasNTP = false;
var winHasInetTime = false;

var syncText = null;
var syncScrollbar = null;
var lastTextVal, lastScrollVal;

var VanadielClock_TimerService = Components.classes["@xenoveritas.org/vanadiel-timer;1"].
               getService(Components.interfaces.nsIVanadielTimer);

function init() {
    var strings = document.getElementById("strings");
    syncText = document.getElementById("sync-text");
    syncScrollbar = document.getElementById("sync-scrollbar");
    // See what OS we're on
    var os = navigator.oscpu;
    if (os.length >= 11) {
	if (os.substring(0,11) == "Windows NT ") {
	    // see if it's 5.1 or greater (supports internet time sync)
	    var ver = parseInt(os.substring(11));
	    if (ver >= 5) {
		winHasNTP = true;
		// add in "sync with Windows" display
		if (ver == 5) {
		    var i = os.indexOf(".", 11);
		    if (i >= 0) {
			var subver = parseInt(os.substring(i+1));
			if (subver >= 1) {
			    winHasInetTime = true;
			}
		    }
		} else {
		    winHasInetTime = true;
		}
		var tab = document.createElement("tab");
		tab.setAttribute("label", strings.getString("windows.internet.time"));
		document.getElementById("infoTabs").appendChild(tab);
		var winntp = document.createElement("tabpanel");
		winntp.setAttribute("orient", "vertical");
		document.getElementById("infoTabpanels").appendChild(winntp);
		var desc = document.createElement("description");
		desc.appendChild(document.createTextNode(strings.getString("windows.internet.time.desc")));
		winntp.appendChild(desc);
		if (winHasInetTime) {
		    desc = document.createElement("description");
		    desc.appendChild(document.createTextNode(strings.getString("win32.ntp.timedate")));
		    winntp.appendChild(desc);
		    desc = document.createElement("description");
		    desc.appendChild(document.createTextNode(strings.getString("win32.ntp.timedate.2")));
		    winntp.appendChild(desc);
		    var hbox = document.createElement("hbox");
		    hbox.setAttribute("pack", "center");
		    winntp.appendChild(hbox);
		    var but = document.createElement("button");
		    but.setAttribute("label", strings.getString("win32.ntp.timedate.open"));
		    but.setAttribute("oncommand", "win32_open_timedate()");
		    hbox.appendChild(but);
		    desc = document.createElement("description");
		    desc.appendChild(document.createTextNode(strings.getString("win32.ntp.timedate.3")));
		    winntp.appendChild(desc);
		}
		var desc = document.createElement("description");
		desc.appendChild(document.createTextNode(strings.getString("win32.ntp.noTimeDate")));
		winntp.appendChild(desc);
		desc = document.createElement("description");
		desc.appendChild(document.createTextNode(strings.getString("win32.ntp.noTimeDate.1")));
		winntp.appendChild(desc);
		desc = document.createElement("description");
		desc.appendChild(document.createTextNode(strings.getString("win32.ntp.noTimeDate.2")));
		winntp.appendChild(desc);
		var group = document.createElement("groupbox");
		winntp.appendChild(group);
		var caption = document.createElement("caption");
		group.appendChild(caption);
		caption.setAttribute("label", strings.getString("win32.ntp.noTimeDate.cmd"));
		desc = document.createElement("textbox");
		desc.setAttribute("rows", "3");
		desc.setAttribute("multiline", "true");
		desc.setAttribute("readonly", "true");
		desc.setAttribute("value",
			strings.getString("win32.ntp.noTimeDate.cmd.1") + "\n" +
			strings.getString("win32.ntp.noTimeDate.cmd.2") + "\n" +
			strings.getString("win32.ntp.noTimeDate.cmd.3"));
		group.appendChild(desc);
		desc = document.createElement("description");
		desc.appendChild(document.createTextNode(strings.getString("win32.ntp.noTimeDate.3")));
		winntp.appendChild(desc);
		tab = document.getElementById("basicInfoTab");
		desc = document.createElement("label");
		desc.setAttribute("class", "header");
		desc.setAttribute("value", strings.getString("win32.moreInfo.header"));
		tab.appendChild(desc);
		desc = document.createElement("description");
		desc.appendChild(document.createTextNode(strings.getString("win32.moreInfo")));
		tab.appendChild(desc);
		desc = document.createElement("label");
		desc.setAttribute("value", strings.getString("windows.internet.time"));
		desc.setAttribute("class", "link");
		desc.setAttribute("onclick", "document.getElementById('infoTabs').selectedIndex = 2");
		tab.appendChild(desc);
	    }
	}
    }
    // Update the scroll bar every .25 seconds (since no event gets fired on
    // scroll/text changes >:( )
    var offset = Components.classes["@xenoveritas.org/vanadiel-timer;1"].
               getService(Components.interfaces.nsIVanadielTimer).realTimeOffset;
    syncText.value = offset / 1000;
    lastTextVal = "";
    setInterval("updateOffset()", 250);
}

function updateOffset() {
    if (syncText.value != lastTextVal) {
	lastTextVal = syncText.value;
	var i = parseInt(lastTextVal);
	if (!isNaN(i)) {
	    lastScrollVal = syncScrollbar.setAttribute("curpos", i + 600);
	    VanadielClock_TimerService.setRealTimeOffset(Math.floor(parseFloat(syncText.value) * 1000));
	}
    } else if (syncScrollbar.getAttribute("curpos") != lastScrollVal) {
	lastScrollVal = syncScrollbar.getAttribute("curpos");
	var i = parseInt(lastScrollVal) - 600;
	syncText.value = i;
	VanadielClock_TimerService.setRealTimeOffset(i * 1000);
	lastTextVal = syncText.value;
	// when we change it this way, blur the stupid text box
	syncText.blur();
    }
}

function win32_open_timedate() {
    var strings = document.getElementById("strings");
    try {
	// attempts to open the Date and Time control panel applet on Windows
	var environment = Components.classes["@mozilla.org/process/environment;1"]
			   .getService(Components.interfaces.nsIEnvironment);
	var sysroot = environment.get("SystemRoot");
	dump("Looking for " + sysroot + "\\System32\\timedate.cpl\n");
	// now try and create a file
	var file = Components.classes["@mozilla.org/file/local;1"]
		    .getService(Components.interfaces.nsILocalFile);
	file.initWithPath(sysroot + "\\System32\\timedate.cpl");
	if (!file.exists()) {
	    alert(strings.getString("win32.ntp.timedate.notFound"));
	} else {
	    file.launch();
	}
    } catch (ex) {
	dump("Error attempting to access Date and Time panel:\n");
	dump(ex);
	dump("\n");
	alert(strings.getString("win32.ntp.timedate.errorOpening"));
    }
}
